==> 0.5.7.tar.gz: lib/base/base_refresh_list_controller.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';
import '../flutter_mylibrary.dart';
import 'base_view_state.dart';

enum LoadType {
  Refresh,
  LoadMore,
}

mixin BaseRefreshListController<T> on BaseViewState{


  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  int _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }



  bool get hasMore => getRequestInfo().useLoadMore?totalNum > list.length? true : false : false;


  onCompleted(List<T> data) {}


  List<T> getDataList(HttpData httpData){
    return null;
  }


  ///---------------------------------------------------数据获取逻辑区-------------------------------------------------------

  void _getData(LoadType loadType) async{

    HttpData httpData = await request((httpData) {
      if(currentPageNum == firstPage()){
        list.clear();
      }

      List<T> resList = [];

      if(httpData.data is List){
        resList = List<T>.from(httpData.data);
      }else{
        resList = getDataList(httpData);
      }

      list.addAll(resList);
    }, isSetError: currentPageNum == firstPage());

    if(isEmpty(httpData.httpError)){
      setIdle();
    }

    bool success = true;

    if(isNotEmpty(httpData.httpError)){
      if(loadType == LoadType.LoadMore){
        _currentPageNum = --_currentPageNum;
      }
      if(httpData.httpError.httpErrorType != HttpErrorType.EmptyDataError){
        success = false;
      }
    }

    refreshController.finishRefresh(success: success);
    refreshController.finishLoad(success: success, noMore: !hasMore);
  }




  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      _getData(LoadType.Refresh);
    }
  }




  ///------------------------------------------------------刷新，上拉加载----------------------------------------------------

  void refreshData() {
    _currentPageNum = firstPage();
    _getData(LoadType.Refresh);
  }


  /// 上拉加载更多
  void loadMore() {
    _currentPageNum = ++_currentPageNum;
    _getData(LoadType.LoadMore);
  }




  ///------------------------------------------------------下拉刷新控件----------------------------------------------------

  Widget getEasyRefreshWidget({ScrollController scrollController, bool useSliverWidget = false, Widget headerWidget, Widget footWidget, Widget emptyWidget, ViewStateModelWidgetListBuilder<List<T>> widgetListBuilder, ViewStateModelWidgetBuilder<List<T>> builder, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true, OnLoadCallback onLoad}){
    Widget emptyErrorWidget = buildRefreshNoDataWidget(viewState: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget, showBusyWidget: showBusyWidget);
    if(isEmpty(headerWidget) && isEmpty(footWidget)){
      return EasyRefresh(
        scrollController: scrollController,
        enableControlFinishRefresh: true,
        enableControlFinishLoad: true,
        emptyWidget: emptyErrorWidget,
        onRefresh: isBusy? null : () async{
          refreshData();
        },
        onLoad: (isBusy || !hasMore)? null : () async{
          loadMore();
        },
        controller: refreshController,
        child: isEmpty(emptyErrorWidget)? builder(list) : Container(),
      );
    }else{
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Expanded(
            child: useSliverWidget? EasyRefresh.custom(
              scrollController: scrollController,
              enableControlFinishRefresh: true,
              enableControlFinishLoad: true,
              emptyWidget: emptyErrorWidget,
              onRefresh: isBusy? null : () async{
                refreshData();
              },
              onLoad: (isBusy || !hasMore)? null : () async{
                loadMore();
              },
              slivers: isBusy || isError? [SliverToBoxAdapter(
                child: Container(),
              )] : isNotEmpty(widgetListBuilder)? widgetListBuilder(list) : [isEmpty(emptyErrorWidget)? isNotEmpty(builder)? builder(list) : SliverToBoxAdapter(child: Container(),) :SliverToBoxAdapter(child: Container(),),],
              controller: refreshController,
            ) : EasyRefresh(
              child: isEmpty(emptyErrorWidget)? builder(list) : Container(),
              scrollController: scrollController,
              enableControlFinishRefresh: true,
              enableControlFinishLoad: true,
              emptyWidget: emptyErrorWidget,
              onRefresh: isBusy? null : () async{
                refreshData();
              },
              onLoad: (isBusy || !hasMore)? null : () async{
                loadMore();
              },
              controller: refreshController,
            ),
          ),
          footWidget??Container(),
        ],
      );
    }
  }



}

==> 0.5.56.tar.gz: lib/base/base_refresh_list_controller.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';
import '../flutter_mylibrary.dart';
import 'base_view_state.dart';

enum LoadType {
  Refresh,
  LoadMore,
}

mixin BaseRefreshListController<T> on BaseViewState{


  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  int _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }



  bool get hasMore => getRequestInfo().useLoadMore?totalNum > list.length? true : false : false;


  onCompleted(List<T> data) {}


  List<T> getDataList(HttpData httpData){
    return null;
  }


  ///---------------------------------------------------数据获取逻辑区-------------------------------------------------------

  void _getData(LoadType loadType) async{

    HttpData httpData = await request((httpData) {
      if(currentPageNum == firstPage()){
        list.clear();
      }

      List<T> resList = [];

      if(httpData.data is List){
        resList = List<T>.from(httpData.data);
      }else{
        resList = getDataList(httpData);
      }

      list.addAll(resList);
    }, isSetError: currentPageNum == firstPage());

    if(isEmpty(httpData.httpError)){
      setIdle();
    }

    bool success = true;

    if(isNotEmpty(httpData.httpError)){
      if(loadType == LoadType.LoadMore){
        _currentPageNum = --_currentPageNum;
      }
      if(httpData.httpError.httpErrorType != HttpErrorType.EmptyDataError){
        success = false;
      }
    }

    refreshController.finishRefresh(success: success);
    refreshController.finishLoad(success: success, noMore: !hasMore);
  }




  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      _getData(LoadType.Refresh);
    }
  }




  ///------------------------------------------------------刷新，上拉加载----------------------------------------------------

  void refreshData() {
    _currentPageNum = firstPage();
    _getData(LoadType.Refresh);
  }


  /// 上拉加载更多
  void loadMore() {
    _currentPageNum = ++_currentPageNum;
    _getData(LoadType.LoadMore);
  }




  ///------------------------------------------------------下拉刷新控件----------------------------------------------------

  Widget getEasyRefreshWidget({
    ScrollController scrollController,
    Widget footWidget,
    Widget emptyWidget,
    ViewStateModelWidgetListBuilder<List<T>> widgetListBuilder,
    ViewStateModelWidgetBuilder<List<T>> widgetBuilder,
    Widget busyWidget,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
    }){
      return buildViewStateEasyRefresh<List<T>>(
        data: list,
        scrollController: scrollController,
        footWidget: footWidget,
        emptyWidget: emptyWidget,
        widgetListBuilder: widgetListBuilder,
        widgetBuilder: widgetBuilder,
        busyWidget: busyWidget,
        showBusyWidget: showBusyWidget,
        showEmptyWidget: showEmptyWidget,
        showErrorWidget: showErrorWidget,
        onLoad: hasMore? onLoad??(){
          loadMore();
        } : null,
        onRefresh: onRefresh??(){
          refreshData();
        }
      );
  }




}
